Param (
[switch]$version = $false
)
$arcconf="C:\ProgramData\checkmk\agent\local\adaptec\arcconf.exe"
function makeobj ($count,$array)
{
    $obj=for ($i = 0; $i -lt ($array | measure -Line).Lines; $i += $count)
    {
        $strings = $array | select -First $count -Skip $i
        $table = @{}
        foreach ($string in $strings)
        {
            if($string -match ":")
            {
                $fstring = $string -split ":" ; 
                $table.add($fstring[0].trim(),$fstring[1].trim())
            }
        }
        New-Object psobject -Property $table
    }
    return $obj
}
function adinfo
{
    $nad=5
    $adprejson=@{}
    $addatapattern='(Controller Status.*|  Temperature.*|Controller Model.*|Status.*ZMM.*|Controller Serial Number.*)'
    $addata=(& $arcconf getconfig 1 ad).split("`n")
    $addata=(($addata | Select-String -Pattern $addatapattern) -split '`n')
    $adinfo=(makeobj($nad)($addata))
    $adinfo | ForEach-Object{
        $i++
        $State=$_."Controller Status"
        $Model=$_."Controller Model"        
        $Temperature="-"
        if($_."Temperature".length -gt 0)
        {
            $Temperature=$_."Temperature"
            if($Temperature -match " ")
            {
                $Temperature=$Temperature.split(" ")[0]
            }
            $Temperature="Temperature=$($Temperature)"
        }
        $Battery=$_."Status"
        if($Battery.length -le 0)
        {
            $Battery="no battery installed"
        }
        $SN=$_."Controller Serial Number"
        if($State -eq "Optimal")
        {
            Write-Host "0 check_adaptec_AD$($i) $($Temperature) Battery state: $($Battery) - Model:$($Model)"
        }
        else
        {
            Write-Host "2 check_adaptec_AD$($i) $($Temperature) State:$($State) - Battery state: $($Battery) - Model:$($Model)"
        }
    }
}
function ldinfo
{ 
    $nld=3
    $ldprejson=@{}
    $lddatapattern='(Logical Device name.*|RAID level.*|Status of Logical Device.*)'
    $lddata=(& $arcconf getconfig 1 ld).split("`n")
    $lddata=(($lddata | Select-String -Pattern $lddatapattern) -split '`n')
    $ldinfo=(makeobj($nld)($lddata))
    $ldinfo | ForEach-Object{
        $ID=$_."Logical Device name"
        $ID=$ID.Replace(" ","_")
        $State=$_."Status of Logical Device"
        $RaidLevel=$_."RAID level"
        if($State -eq "Optimal")
        {
            Write-Host "0 check_adaptec_$($ID) - RaidLevel:$($RaidLevel)"
        }
        else
        {
            Write-Host "2 check_adaptec_$($ID) - State:$($State) - RaidLevel:$($RaidLevel)"
        }

    }
}
function pdinfo
{
    $pddata=((& $arcconf getconfig 1 pd) -replace ">", "" -split '`n\s*')#.split("`n\s*")
    $ID=""
    $State=""
    $Serialnumber=""
    $Model=""
    $Temperature=""
    $SMARTwarnings=""
    ForEach($data in $pddata)
    {
        $data = $data.Trim()
        If ($data -match '----------------------------------------------------------------' -and $Out -eq $True) 
        {
            $Out = $False
            if($Temperature.length -gt 0)
            {
                if($Temperature -match " ")
                {
                    $Temperature=$Temperature.split(" ")[0]
                }
                $Temperature="|Temperature=$($Temperature)"
            }
            $ID=$ID.Replace(" ","_")
            if($State -eq "Online")
            {
                Write-Host "0 check_adaptec_PD$($ID) SMARTwarnings=$($SMARTwarnings)$($Temperature) Model: $($Model) - Serialnumber: $($Serialnumber)"
            }
            else
            {
                Write-Host "2 check_adaptec_PD$($ID) SMARTwarnings=$($SMARTwarnings)$($Temperature) State:$($State) - Model: $($Model) - Serialnumber: $($Serialnumber)"
            }
        }

        If ($Out -eq $True) 
        {
            If ($data -match '^State\s*:') {$fstring = $data -split "\s:" ; $State = $fstring[1].trim()}
            If ($data -match '^Reported Channel.Device.T.L.\s*:') {$fstring = $data -split "\s:" ; $ID = $fstring[1].trim()}
            If ($data -match '^Serial number\s*:') {$fstring = $data -split "\s:" ; $Serialnumber = $fstring[1].trim()}
            If ($data -match '^Model\s*:') {$fstring = $data -split "\s:" ; $Model = $fstring[1].trim()}
            If ($data -match '^Temperature\s*:') {$fstring = $data -split "\s:" ; $Temperature = $fstring[1].trim()}
            If ($data -match '^S.M.A.R.T. warnings\s*:') {$fstring = $data -split "\s:" ; $SMARTwarnings = $fstring[1].trim()}
        }
        If ($data -match 'Device is a Hard drive') 
        {
            $Out = $True
            $ID=""
            $State=""
            $Serialnumber=""
            $Model=""
            $Temperature=""
            $SMARTwarnings=""
        }
    }
}
function pdlld
{
    $npd=2
    $pdprejson=@{}
    $pddatapattern='(Reported Channel.*|Serial number.*)'
    $pddata=((& $arcconf getconfig 1 pd | select-string -Pattern "Device is a Hard drive" -Context 0,12) -replace ">", "").split("`n")
    $pddata=(($pddata | Select-String -Pattern $pddatapattern) -split '`n')
    $pdinfo=(makeobj($npd)($pddata))
    $pds=(($pdinfo | Select-Object -Property 'Reported Channel,Device(T:L)','Serial number') | convertto-json)
    $pds = "{"+'"data"'+":"+((($pds -ireplace '("Reported Channel,Device\(T:L\)")','"{#SLOT}"') -ireplace 'Serial number','{#SN}'))+"}"
    return $pds
}
function ldlld
{ 
    $nld=1
    $ldprejson=@{}
    $lddatapattern='(Logical Device name.*)'
    $lddata=(& $arcconf getconfig 1 ld).split("`n")
    $lddata=(($lddata | Select-String -Pattern $lddatapattern) -split '`n')
    $ldinfo=(makeobj($nld)($lddata))
    $lds=(($ldinfo | Select-Object -Property 'Logical Device name') | convertto-json)
    $lds = "{"+'"data"'+":"+ "["+(($lds -ireplace 'Logical Device name','{#LDNAME}'))+"]}"
    return $lds
}

adinfo
ldinfo
pdinfo
